const path = require('path');
const { readFile, writeFile } = require('fs-extra');
const getWritableDirectory = require('@now/build-utils/fs/get-writable-directory.js');
const download = require('@now/build-utils/fs/download.js');
const FileBlob = require('@now/build-utils/file-blob.js');
const glob = require('@now/build-utils/fs/glob.js');
const { createLambda } = require('@now/build-utils/lambda.js');
const { commandInDir } = require('./cli');

const downloadCompileRuby = require('./download-compile-ruby');
const installRubyGems = require('./install-rubygems');

exports.config = {
  maxLambdaSize: '50mb',
};

async function getRubyVersion(files) {
  let versionFile = (files['.ruby-version'] || files['Gemfile']);
  if (!versionFile) return versionFile; // versionFile is undefined
  // different version spec format per file
  // Example
  // Gemfile: ruby '2.5.1'
  // .ruby-version: 'ruby-2.5.1'
  versionFile = await FileBlob.fromStream({stream: versionFile.toStream()})
  const versionNumber = versionFile.data
    .toString()
    .match(/ruby(?:\s|-)(\d\.\d\.\d)/)
    // Works on Node v10.7.0 but not on `now`
    // SyntaxError: Invalid regular expression: /ruby(?:\s|-)(?<version>\d\.\d\.\d)/: Invalid group
    // .match(/ruby(?:\s|-)(?<version>\d\.\d\.\d)/);
  // return versionNumber ? versionNumber.groups.version : undefined; // null guard
  return versionNumber ? versionNumber[1] : undefined;
}

function checkForUserSpecifiedHandler(files) {
  return Object.keys(files)
    .filter(key => ['handler.rb', 'lambda.rb', 'now_handler.rb'].includes(key))
    .shift();
}

exports.build = async ({ files, entrypoint }) => {
  console.log('downloading files...');

  const srcPath = await getWritableDirectory();

  const allFiles = await download(files, srcPath);


  if (allFiles["Gemfile"]) {
    console.log("'Gemfile' is present in your app......");

    console.log('Installing Ruby.....');
    let rubyVersion = getRubyVersion(allFiles); // || '2.6.1';
    await downloadCompileRuby(await rubyVersion);

    // eslint-disable-next-line no-unused-vars
    // const [_, installedRubyVersion] = rubyVersion.match(/([0-9]\.[0-9]\.[0-9])$/);
    // console.log('installed version ' + installedRubyVersion);
    const gemfile = allFiles["Gemfile"].fsPath;
    const bundledGems = await installRubyGems(srcPath, '--deployment');
    // console.log(bundledGems)
    // console.dir(bundledGems)
  }

  if (allFiles['build.sh']) {
    await commandInDir(srcPath)('bash', allFiles['build.sh'].fsPath);
  }

  const handlerFile = checkForUserSpecifiedHandler(allFiles);
  console.log('handlerFile = ' + handlerFile)
  let nowHandlerRbFilename = '';

  if (!handlerFile) {
    const handlerForRubyFiles = await readFile(
      path.join(__dirname, 'now_handler.rb'),
      'utf8',
    );

    // replace all `require_relative 'now_handler'`
    console.log('entrypoint is', entrypoint);
    const userDefinedHandlerPath = entrypoint
      .replace(/\//g, '.')
      .replace(/\.rb$/, '');
    const nowHandlerRbContents = handlerForRubyFiles.replace(
      '__NOW_HANDLER_FILE',
      userDefinedHandlerPath,
    );

    // in order to allow the user to have `now_handler.rb`,
    // we need our `now_handler.rb` to be called
    // somethig else
    nowHandlerRbFilename = 'now__handler__ruby';
    const entrypointDirname = path.dirname(allFiles[entrypoint].fsPath);

    await writeFile(
      path.join(entrypointDirname, `${nowHandlerRbFilename}.rb`),
      nowHandlerRbContents,
    );
  }

  console.log(await glob('**', srcPath))

  const lambda = await createLambda({
    files: await glob('**', srcPath),
    handler: ( handlerFile ? 
      `${handlerFile}.handler` : 
      `${nowHandlerRbFilename}.NowHandler.now_handler`
    ),
    runtime: 'ruby2.5',
    environment: {},
  });

  return {
    [entrypoint]: lambda,
  };
};
